import logging
try:  # Python 2.7+
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):

        def emit(self, record):
            pass

logging.getLogger('oauthlib').addHandler(NullHandler())



# -*- coding: utf-8 -*-
from django.conf import settings as django_settings
import random
import hashlib
from urllib import quote
from .models import OauthApps
import json


def createNoncestr(length=32):
    """产生随机字符串，不长于32位"""
    chars = "abcdefghijklmnopqrstuvwxyz0123456789"
    strs = []
    for x in range(length):
        strs.append(chars[random.randrange(0, len(chars))])
    return "".join(strs)


def formatBizQueryParaMap(paraMap, urlencode):
    """格式化参数，签名过程需要使用"""
    slist = sorted(paraMap)
    buff = []
    for k in slist:
        v = paraMap[k]
        if not isinstance(paraMap[k], (int, str)):
            v = json.dumps(paraMap[k])
        v = quote(v) if urlencode else v
        buff.append("{0}={1}".format(k, v))
    return "&".join(buff)


def getSign(obj, secret):
    """生成签名"""
    # 签名步骤一：按字典序排序参数,formatBizQueryParaMap已做
    String = formatBizQueryParaMap(obj, False)
    # 签名步骤二：在string后加入KEY
    String = "{0}&secret={1}".format(String, secret)
    # 签名步骤三：MD5加密
    String = hashlib.md5(String).hexdigest()

    # 签名步骤四：所有字符转为大写
    result_ = String.upper()
    return result_


def verifySign(obj, secret, signature):
    return signature == getSign(obj, secret)


def get_sign_key(app):
    if hasattr(django_settings, "DJDG_AUTH"):
        auth_list = [
            x for x in django_settings.DJDG_AUTH.get(
                "APPS", []) if x["app"] == app]
        return auth_list[0] if auth_list else None
    else:
        return None


def get_verify_key(appid):
    auth_list = OauthApps.objects.filter(appid=appid)
    return auth_list[0] if auth_list else None


get_verify_key(80145171)


def set_parameters(parameters, app):
    appkeys = get_sign_key(app)
    if not appkeys:
        raise Exception("app secret not find, please check you settings file")
    parameters["appid"] = appkeys["appid"]
    parameters["nonce_str"] = createNoncestr(12)
    return getSign(parameters, appkeys["secret"]), parameters



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from djdg_oauth.models import OauthApps


class Command(BaseCommand):
    help = 'list, add or delete an app auth'

    def add_arguments(self, parser):
        parser.add_argument('app', nargs='+', type=str)

    def handle(self, *args, **options):
        apps = options["app"]
        for app in apps:
            self.stdout.write("\n=====================\n")
            auth_keys = OauthApps.objects.filter(app=app)
            if not auth_keys:
                self.stdout.write(
                    "app {app} auth key not found\n".format(
                        app=app))
                continue
            auth_keys.delete()
            self.stdout.write(
                "app {app} auth key delete success!".format(
                    app=app))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
import requests
try:
    from urlparse import urlparse, urlunparse
except ImportError:
    from urllib.parse import urlparse, urlunparse
from .oauthcore import get_verify_key, verifySign, set_parameters
import logging
log = logging.getLogger("oauthlib")


class OAuthClient(object):
    """
    TODO: add docs
    """
    def __init__(self):
        """
        :params server: An instance of oauthlib.oauth2.Server class
        """
        pass

    def _get_escaped_full_path(self, request):
        """
        Django considers "safe" some characters that aren't so for oauthlib.
        We have to search for them and properly escape.
        """
        parsed = list(urlparse(request.get_full_path()))
        return urlunparse(parsed)

    def _extract_params(self, request):
        """
        """
        uri = self._get_escaped_full_path(request)
        http_method = request.method
        headers = self.extract_headers(request)
        body = dict(self.extract_body(request))
        return uri, http_method, body, headers

    def extract_headers(self, request):
        """
        Extracts headers from the Django request object
        :param request: The current django.http.HttpRequest object
        :return: a dictionary with OAuthLib needed headers
        """
        headers = request.META.copy()
        if 'HTTP_AUTHORIZATION' in headers:
            headers['Authorization'] = headers['HTTP_AUTHORIZATION']
        return headers

    def extract_body(self, request):
        """
        Extracts the POST body from the Django request object
        :param request: The current django.http.HttpRequest object
        :return: provided POST parameters
        """
        if request.environ.get("Content-Type") == "application/json":
            return json.loads(request.body.decode('utf-8')).items()
        elif request.method == "GET":
            return request.GET.items()
        else:
            return request.POST.items()

    def verify_request(self, request):
        """
        :param request: The current django.http.HttpRequest object
        """
        uri, http_method, body, headers = self._extract_params(request)
        if not body.get("appid"):
            raise Exception(u"can not find appid in request body")
        keys_obj = get_verify_key(body.get("appid"))
        if not keys_obj:
            raise Exception(
                "can not fetch any settings, please check appid correct")
        signature = headers.get("Authorization")
        request.app_type = keys_obj.app
        return verifySign(body, keys_obj.secret, signature)

    @staticmethod
    def oauth_request(url, method, app, parameters={}, headers=None):
        try:
            signature, parameters = set_parameters(parameters, app)
        except Exception as e:
            log.info(e.message)
            return {"statusCode": 500, "msg": e.message}
        headers_dict = {
            "Accept": "application/json",
            "Content-Type": "application/json;charset=utf-8",
            "Authorization": signature
        }
        headers = headers.update(headers_dict) if headers else headers_dict
        log.info(parameters)
        if method == "get":
            r = requests.request(
                method=method, url=url, headers=headers, params=parameters)
        else:
            r = requests.request(
                method=method, url=url, json=parameters, headers=headers)
        if r.status_code != 200:
            log.error(r.content)
            return {"statusCode": 500, "msg": r.content}
        else:
                try:
                    log.info(r.json())
                    return {"statusCode": 0, "content": r.json()}
                except:
                    log.info(r.content)
                    return {"statusCode": 0, "content": r.content}



# -*- coding: utf-8 -*-
import random
# import uuid
UNICODE_ASCII_CHARACTER_SET = ('abcdefghijklmnopqrstuvwxyz'
                               'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
                               '0123456789')
NUMBERIC_ASCII_CHARACTER_SET = ('0123456789')


def gen_app_id():
    return generate_token(8, NUMBERIC_ASCII_CHARACTER_SET)


def gen_secret():
    return generate_token(40, UNICODE_ASCII_CHARACTER_SET)


def gen_sid():
    return generate_token(16, NUMBERIC_ASCII_CHARACTER_SET)


def generate_token(length=30, chars=UNICODE_ASCII_CHARACTER_SET):
    """Generates a non-guessable OAuth token

    OAuth (1 and 2) does not specify the format of tokens except that they
    should be strings of random characters. Tokens should not be guessable
    and entropy when generating the random characters is important. Which is
    why SystemRandom is used instead of the default random.choice method.
    """
    rand = random.SystemRandom()
    return ''.join(rand.choice(chars) for x in range(length))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-08-24 08:42
from __future__ import unicode_literals

from django.db import migrations, models
import djdg_oauth.generator


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='OauthApps',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('appid', models.CharField(default=djdg_oauth.generator.gen_app_id, max_length=40)),
                ('secret', models.CharField(default=djdg_oauth.generator.gen_secret, max_length=256)),
                ('app', models.CharField(max_length=128)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from django.conf import settings as django_settings
from .oauthclient import OAuthClient
from django.http.response import HttpResponse
import re


class Http401Response(HttpResponse):

    status_code = 401


class DjdgAuthMiddleware(object):

    def process_request(self, request):
        if hasattr(django_settings, "DJDG_AUTH"):
            full_escape_url = django_settings.DJDG_AUTH.get(
                'FULL_ESCAPE_URL', [])
            regex_escape_url = django_settings.DJDG_AUTH.get(
                'REGEX_ESCAPE_URL', [])
            regex_check_url = django_settings.DJDG_AUTH.get(
                'REGEX_CHECK_URL', [])

            path = request.path.strip('/')
            if regex_check_url:
                for url in regex_check_url:
                    if re.match(url, path):
                        break
                else:
                    return
            else:
                if path in full_escape_url:
                    return
                for url in regex_escape_url:
                    if re.match(url, path):
                        return
        try:
            auth = OAuthClient()
            if not auth.verify_request(request):
                return Http401Response('Unauthorized')
        except Exception as e:
            return Http401Response(e.message)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

version = '0.0.6'


LONG_DESCRIPTION = open('README.rst').read()

setup(
    name="djdg-django-oauth",
    version=version,
    description="djdg OAuth2 method for Django",
    long_description=LONG_DESCRIPTION,
    classifiers=[
        "Development Status :: 1 - Planning",
        "Environment :: Web Environment",
        "Framework :: Django :: 1.9",
        "License :: OSI Approved :: BSD License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords='djdg django oauth2 oauthlib',
    author="zaneYang",
    author_email='zane.yang@hey900.com',
    license='BSD',
    packages=find_packages(),
    install_requires=[
        'django==1.9',
        'six',
    ],
    zip_safe=False,
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from .generator import gen_app_id, gen_secret

# Create your models here.


class OauthApps(models.Model):
    """
    appliction's secret and app key list
    """
    appid = models.CharField(max_length=40, default=gen_app_id)
    secret = models.CharField(max_length=256, default=gen_secret)
    app = models.CharField(max_length=128)

    def __str__(self):
        return self.app



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from djdg_oauth.models import OauthApps


class Command(BaseCommand):
    help = 'list, add or delete an app auth'

    def add_arguments(self, parser):
        parser.add_argument('app', nargs='+', type=str)

    def handle(self, *args, **options):
        apps = options["app"]
        for app in apps:
            self.stdout.write("\n=====================\n")
            auth_keys = OauthApps.objects.filter(app=app)
            if auth_keys:
                self.stdout.write(
                    "app {app} auth key already exists,".format(app=app) +
                    " use \'listauthapp\' to check.\n")
                continue
            _auth = OauthApps(app=app)
            _auth.save()
            auth_str = "appid: " + str(_auth.appid) +\
                "\t secret: " + str(_auth.secret)
            self.stdout.write(
                "add app {app} auth key Succeed:\n {auth_str}".format(
                    app=app, auth_str=auth_str))



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from djdg_oauth.models import OauthApps


class Command(BaseCommand):
    help = 'list, add or delete an app auth'

    def add_arguments(self, parser):
        parser.add_argument('app', nargs='+', type=str)
        # parser.add_argument(
        #     '--l',
        #     dest='list',
        #     default='list',
        #     type=str,
        #     help='List secret info of the input app ',
        # )
        # parser.add_argument(
        #     '--a',
        #     dest='add',
        #     default='add',
        #     type=str,
        #     help='add secret info of the input app ',
        # )
        # parser.add_argument(
        #     '--d',
        #     dest='delete',
        #     default='delete',
        #     type=str,
        #     help='delete secret info of the input app ',
        # )

    def handle(self, *args, **options):
        apps = options["app"]
        for app in apps:
            self.stdout.write("=====================\n")
            auth_keys = OauthApps.objects.filter(app=app)
            if not auth_keys:
                self.stdout.write(
                    "app {app} auth key not found\n".format(
                        app=app),)
                continue
            else:
                auth_key = auth_keys[0]
                auth_str = "appid: " + str(auth_key.appid) +\
                    "\t secret: " + str(auth_key.secret)
                self.stdout.write("app {app} auth key:\n {auth_str}\n".format(
                    app=app, auth_str=auth_str),)
